Package: NBAMSeq
Type: Package
Title: Negative Binomial Additive Model for RNA-Seq Data
Version: 1.28.0
Authors@R: c(
    person("Xu", "Ren", 
        email = "xuren2120@gmail.com", role = c("aut", "cre")),
    person("Pei Fen", "Kuan", 
        email = "peifen.kuan@stonybrook.edu", role = "aut"))
Description: High-throughput sequencing experiments followed by differential 
    expression analysis is a widely used approach to detect genomic biomarkers. 
    A fundamental step in differential expression analysis is to model the 
    association between gene counts and covariates of interest. NBAMSeq a 
    flexible statistical model based on the generalized additive model and 
    allows for information sharing across genes in variance estimation. 
License: GPL-2
URL: https://github.com/reese3928/NBAMSeq
BugReports: https://github.com/reese3928/NBAMSeq/issues
Encoding: UTF-8
Imports: DESeq2, mgcv(>= 1.8-24), BiocParallel, genefilter, methods,
        stats,
Depends: R (>= 3.6), SummarizedExperiment, S4Vectors
Suggests: knitr, rmarkdown, testthat, ggplot2
RoxygenNote: 6.1.0
VignetteBuilder: knitr
biocViews: RNASeq, DifferentialExpression, GeneExpression, Sequencing,
        Coverage
git_url: https://git.bioconductor.org/packages/NBAMSeq
git_branch: RELEASE_3_23
git_last_commit: 7526963
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:34:35 UTC; biocbuild
Author: Xu Ren [aut, cre],
  Pei Fen Kuan [aut]
Maintainer: Xu Ren <xuren2120@gmail.com>
